package ${basePackage}.module.system.mgr;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.github.pagehelper.util.StringUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import ${basePackage}.frame.base.ErrorType;
import ${basePackage}.frame.base.SortType;
import ${basePackage}.frame.base.Token;
import ${basePackage}.frame.utils.IDgenerator;
import ${basePackage}.frame.utils.MapperUtil;
import ${basePackage}.frame.utils.Message;
import ${basePackage}.frame.utils.ValidationUtil;
import ${basePackage}.module.system.ent.Resource;
import ${basePackage}.module.system.ent.ResourceCheck;
import ${basePackage}.module.system.ent.ResourceTree;
import ${basePackage}.module.system.mpr.ResourceMapper;
import ${basePackage}.module.system.req.ResourceCreateRequest;
import ${basePackage}.module.system.req.ResourceDeleteRequest;
import ${basePackage}.module.system.req.ResourceFindRequest;
import ${basePackage}.module.system.req.ResourceGetRequest;
import ${basePackage}.module.system.req.ResourceLoadRequest;
import ${basePackage}.module.system.req.ResourceUpdateRequest;
import ${basePackage}.module.system.rsp.ResourceCreateResponse;
import ${basePackage}.module.system.rsp.ResourceDeleteResponse;
import ${basePackage}.module.system.rsp.ResourceFindResponse;
import ${basePackage}.module.system.rsp.ResourceGetResponse;
import ${basePackage}.module.system.rsp.ResourceLoadResponse;
import ${basePackage}.module.system.rsp.ResourceUpdateResponse;

import java.util.ArrayList;
import java.util.List;

/**
 * RESOURCE - 资源
 *
 * @author author
 * @version 0.0.1
 * @since 2017-01-01
 */
@Transactional
@Service
public class ResourceManagerImpl implements ResourceManager {

    @Autowired
    private ResourceMapper resourceMapper;
    @Autowired
    private DictItemManager dictItemManager;

    /**
     * 插入
     *
     * @param request 请求对象
     * @param token   令牌
     * @return 响应
     */
    public ResourceCreateResponse create(ResourceCreateRequest request, Token token) {
        ResourceCreateResponse response = new ResourceCreateResponse();

        ValidationUtil.validate(request, response);
        if (response.hasError()) {
            return response;
        }

        long id = IDgenerator.nextId();
        Resource entity = MapperUtil.map(request, Resource.class);
        entity.setId(id);

        long result = resourceMapper.insert(entity, token);
        if (1L != result) {
            response.addError(ErrorType.BUSINESS_ERROR, Message.CREATE_FAILURE);
            return response;
        }
        response.setId(id);

        return response;
    }

    /**
     * 逻辑删除
     *
     * @param request 请求对象
     * @param token   令牌
     * @return 响应
     */
    public ResourceDeleteResponse delete(ResourceDeleteRequest request, Token token) {
        ResourceDeleteResponse response = new ResourceDeleteResponse();

        ValidationUtil.validate(request, response);
        if (response.hasError()) {
            return response;
        }

        long result = resourceMapper.delete(request, token);
        if (1L != result) {
            response.addError(ErrorType.BUSINESS_ERROR, Message.DELETE_FAILURE);
            return response;
        }
        response.setResult(result);

        return response;
    }

    /**
     * 更新
     *
     * @param request 请求对象
     * @param token   令牌
     * @return 响应
     */
    public ResourceUpdateResponse update(ResourceUpdateRequest request, Token token) {
        ResourceUpdateResponse response = new ResourceUpdateResponse();

        ValidationUtil.validate(request, response);
        if (response.hasError()) {
            return response;
        }

        long result = resourceMapper.update(request, token);
        if (1L != result) {
            response.addError(ErrorType.BUSINESS_ERROR, Message.UPDATE_FAILURE);
            return response;
        }
        response.setResult(result);

        return response;
    }

    /**
     * 查询
     *
     * @param request 请求对象
     * @param token   令牌
     * @return 响应
     */
    @Transactional(readOnly = true)
    public ResourceFindResponse find(ResourceFindRequest request, Token token) {
        ResourceFindResponse response = new ResourceFindResponse();

        ValidationUtil.validate(request, response);
        if (response.hasError()) {
            return response;
        }

        PageHelper.startPage(request.getPageNumber(), request.getPageSize());
        if (StringUtil.isNotEmpty(request.getSortKey())) {
            PageHelper.orderBy(request.getSortKey() + " " + request.getSortType());
        }
        PageInfo<Resource> pageInfo = new PageInfo<>(resourceMapper.find(request, token));

        response.setResult(pageInfo.getList());
        response.setTotalCount(pageInfo.getTotal());

        return response;
    }

    /**
     * 获得对象
     *
     * @param request 请求对象
     * @param token   令牌
     * @return 响应
     */
    @Transactional(readOnly = true)
    public ResourceGetResponse get(ResourceGetRequest request, Token token) {
        ResourceGetResponse response = new ResourceGetResponse();

        ValidationUtil.validate(request, response);
        if (response.hasError()) {
            return response;
        }

        Resource po = resourceMapper.get(request, token);

        if (po != null) {
            response.setResource(po);
        } else {
            response.addError(ErrorType.BUSINESS_ERROR, Message.GET_FAILURE);
        }

        return response;
    }

    @Override
    public ResourceTreeResponse load(ResourceTreeRequest request, Token token) {
        ResourceTreeResponse response = new ResourceTreeResponse();

        ValidationUtil.validate(request, response);
        if (response.hasError()) {
            return response;
        }

        ResourceFindRequest resourceFindRequest = new ResourceFindRequest();
        resourceFindRequest.setPageSize(0);
        resourceFindRequest.setValid(true);
        resourceFindRequest.setSortType(SortType.ASC);
        resourceFindRequest.setSortKey("CODE");
        ResourceFindResponse resourceFindResponse = this.find(resourceFindRequest, token);

        if (resourceFindResponse.hasError()) {
            response.addErrors(resourceFindResponse.getErrors());
            return response;
        }

        Map<String, ResourceTree> map = new HashMap<>();
        List<Resource> result = resourceFindResponse.getResult();
        // 获取最顶级资源

        int level = 1;
        while (result.size() > 0) {
            Iterator<Resource> iterator = result.iterator();
            while (iterator.hasNext()) {
                Resource next = iterator.next();
                if (next.getCode() != null && next.getCode().split("_").length == level) {
                    ResourceTree tree = MapperUtil.map(next, ResourceTree.class);
                    map.put(tree.getCode(), tree);
                    iterator.remove();
                    {//尝试获取上级资源
                        String code = next.getCode();
                        ResourceTree Ptree = null;
                        while (code.contains("_")) {
                            code = code.substring(0, code.lastIndexOf("_"));
                            Ptree = map.get(code);
                            if (Ptree != null) break;
                        }
                        if (Ptree != null){
                            Ptree.getChildren().add(tree);
                        } else {
                            map.put(tree.getCode(), tree);
                        }
                    }
                }
            }
            level++;
        }

        return response;
    }
}
